@@ -1,10 +1,14 @@
 #include "config.hpp"
+#ifdef __MINGW32__
+#  define __USE_MINGW_ANSI_STDIO 1
+#endif
 #include "R_interface_sampler.hpp"
 
 #include <cstddef>
+#include <cstdio>  // sprintf
 #include <cstring> // strcmp, memcpy
 
-#include <external/alloca.h>
+#include <misc/alloca.h>
 
 #include <R_ext/Random.h> // GetRNGstate, PutRNGState
 
@@ -20,8 +24,6 @@
 #include "R_interface.hpp"
 #include "R_interface_common.hpp"
 
-#define asRXLen(_X_) static_cast<R_xlen_t>(_X_)
-
 using std::size_t;
 using namespace dbarts;
 
@@ -75,10 +77,10 @@ extern "C" {
     int i_temp;
     size_t numBurnIn, numSamples;
     
-    i_temp = rc_getInt(numBurnInExpr, "number of burn-in steps", RC_LENGTH | RC_GEQ, asRXLen(1), RC_VALUE | RC_GEQ, 0, RC_NA | RC_YES, RC_END);
+    i_temp = rc_getInt(numBurnInExpr, "number of burn-in steps", RC_LENGTH | RC_GEQ, rc_asRLength(1), RC_VALUE | RC_GEQ, 0, RC_NA | RC_YES, RC_END);
     numBurnIn = i_temp == NA_INTEGER ? fit->control.defaultNumBurnIn : static_cast<size_t>(i_temp);
     
-    i_temp = rc_getInt(numSamplesExpr, "number of samples", RC_LENGTH | RC_GEQ, asRXLen(1), RC_VALUE | RC_GEQ, 0, RC_NA | RC_YES, RC_END);    
+    i_temp = rc_getInt(numSamplesExpr, "number of samples", RC_LENGTH | RC_GEQ, rc_asRLength(1), RC_VALUE | RC_GEQ, 0, RC_NA | RC_YES, RC_END);    
     numSamples = i_temp == NA_INTEGER ? fit->control.defaultNumSamples : static_cast<size_t>(i_temp);
     
     if (numBurnIn == 0 && numSamples == 0) Rf_error("either number of burn-in or samples must be positive");
@@ -86,14 +88,14 @@ extern "C" {
     size_t numTrainingSamples = fit->data.numObservations * numSamples;
     if (numSamples != 0 && numTrainingSamples / numSamples != fit->data.numObservations)
       Rf_error("training sample array size exceeds architecture's capacity");
-    R_xlen_t s_numTrainingSamples = asRXLen(numTrainingSamples);
+    R_xlen_t s_numTrainingSamples = rc_asRLength(numTrainingSamples);
     if (s_numTrainingSamples < 0 || static_cast<size_t>(s_numTrainingSamples) != numTrainingSamples)
       Rf_error("training sample array size cannot be represented by a signed integer on this architecture");
     
     size_t numTestSamples = fit->data.numTestObservations * numSamples;
      if (numSamples != 0 && numTestSamples / numSamples != fit->data.numTestObservations)
       Rf_error("test sample array size exceeds architecture's capacity");
-    R_xlen_t s_numTestSamples = asRXLen(numTestSamples);
+    R_xlen_t s_numTestSamples = rc_asRLength(numTestSamples);
     if (s_numTestSamples < 0 || static_cast<size_t>(s_numTestSamples) != numTestSamples)
       Rf_error("test sample array size cannot be represented by a signed integer on this architecture");
     
@@ -108,15 +110,17 @@ extern "C" {
     
     int protectCount = 0;
     
-    SEXP resultExpr = PROTECT(rc_newList(4));
+    SEXP resultExpr = PROTECT(rc_newList(bartResults->kSamples == NULL ? 4 : 5));
     ++protectCount;
-    SET_VECTOR_ELT(resultExpr, 0, rc_newNumeric(asRXLen(bartResults->getNumSigmaSamples())));
-    SET_VECTOR_ELT(resultExpr, 1, rc_newNumeric(asRXLen(bartResults->getNumTrainingSamples())));
+    SET_VECTOR_ELT(resultExpr, 0, rc_newReal(rc_asRLength(bartResults->getNumSigmaSamples())));
+    SET_VECTOR_ELT(resultExpr, 1, rc_newReal(rc_asRLength(bartResults->getNumTrainingSamples())));
     if (fit->data.numTestObservations > 0)
-      SET_VECTOR_ELT(resultExpr, 2, rc_newNumeric(asRXLen(bartResults->getNumTestSamples())));
+      SET_VECTOR_ELT(resultExpr, 2, rc_newReal(rc_asRLength(bartResults->getNumTestSamples())));
     else
       SET_VECTOR_ELT(resultExpr, 2, R_NilValue);
-    SET_VECTOR_ELT(resultExpr, 3, rc_newInteger(asRXLen(bartResults->getNumVariableCountSamples())));
+    SET_VECTOR_ELT(resultExpr, 3, rc_newInteger(rc_asRLength(bartResults->getNumVariableCountSamples())));
+    if (bartResults->kSamples != NULL)
+      SET_VECTOR_ELT(resultExpr, 4, rc_newReal(rc_asRLength(bartResults->getNumSigmaSamples())));
     
     SEXP sigmaSamples = VECTOR_ELT(resultExpr, 0);
     if (fit->control.numChains > 1)
@@ -149,15 +153,23 @@ extern "C" {
     // these likely need to be down-sized from 64 to 32 bits
     for (size_t i = 0; i < length; ++i) variableCountStorage[i] = static_cast<int>(bartResults->variableCountSamples[i]);
     
+    if (bartResults->kSamples != NULL) {
+      SEXP kSamples = VECTOR_ELT(resultExpr, 4);
+      if (fit->control.numChains > 1)
+        rc_setDims(kSamples, static_cast<int>(bartResults->numSamples), static_cast<int>(fit->control.numChains), -1);
+      std::memcpy(REAL(kSamples), const_cast<const double*>(bartResults->kSamples), bartResults->getNumSigmaSamples() * sizeof(double));
+    }
         
     // create result storage and make it user friendly
     SEXP namesExpr;
     
-    rc_setNames(resultExpr, namesExpr = rc_newCharacter(4));
+    rc_setNames(resultExpr, namesExpr = rc_newCharacter(bartResults->kSamples == NULL ? 4 : 5));
     SET_STRING_ELT(namesExpr, 0, Rf_mkChar("sigma"));
     SET_STRING_ELT(namesExpr, 1, Rf_mkChar("train"));
     SET_STRING_ELT(namesExpr, 2, Rf_mkChar("test"));
     SET_STRING_ELT(namesExpr, 3, Rf_mkChar("varcount"));
+    if (bartResults->kSamples != NULL)
+      SET_STRING_ELT(namesExpr, 4, Rf_mkChar("k"));
     
     UNPROTECT(protectCount);
     
@@ -241,6 +253,15 @@ extern "C" {
     
     Model oldModel = fit->model;
     
+    if ((model.kPrior != NULL && oldModel.kPrior == NULL) ||
+        (model.kPrior == NULL && oldModel.kPrior != NULL))
+    {
+      Rf_error("k prior cannot be changed after sampler has been created");
+      invalidateModel(model);
+      
+      return R_NilValue;
+    }
+    
     fit->setModel(model);
     
     invalidateModel(oldModel);
@@ -296,9 +317,9 @@ extern "C" {
   SEXP setResponse(SEXP fitExpr, SEXP y)
   {
     BARTFit* fit = static_cast<BARTFit*>(R_ExternalPtrAddr(fitExpr));
-    if (fit == NULL) Rf_error("dbarts_setY called on NULL external pointer");
+    if (fit == NULL) Rf_error("dbarts_setResponse called on NULL external pointer");
     
-    rc_assertDoubleConstraints(y, "y", RC_LENGTH | RC_EQ, asRXLen(fit->data.numObservations), RC_END);
+    rc_assertDoubleConstraints(y, "y", RC_LENGTH | RC_EQ, rc_asRLength(fit->data.numObservations), RC_END);
     
     // for binary responses, updates latents and samples
     if (fit->control.responseIsBinary) GetRNGstate();
@@ -333,27 +354,145 @@ extern "C" {
     return R_NilValue;
   }
   
-  SEXP setPredictor(SEXP fitExpr, SEXP x)
+  SEXP setWeights(SEXP fitExpr, SEXP weights)
+  {
+    BARTFit* fit = static_cast<BARTFit*>(R_ExternalPtrAddr(fitExpr));
+    if (fit == NULL) Rf_error("dbarts_setWeights called on NULL external pointer");
+    
+    rc_assertDoubleConstraints(weights, "weights", RC_LENGTH | RC_EQ, rc_asRLength(fit->data.numObservations),
+                               RC_VALUE | RC_GEQ, 0.0,
+                               RC_END);
+    fit->setWeights(REAL(weights));
+    
+    return R_NilValue;
+  }
+  
+  SEXP setPredictor(SEXP fitExpr, SEXP xExpr, SEXP forceUpdateExpr, SEXP updateCutPointsExpr)
   {
     BARTFit* fit = static_cast<BARTFit*>(R_ExternalPtrAddr(fitExpr));
     if (fit == NULL) Rf_error("dbarts_setPredictor called on NULL external pointer");
     
-    if (!Rf_isReal(x)) Rf_error("x must be of type real");
+    if (!Rf_isReal(xExpr)) Rf_error("x must be of type real");
     
-    rc_assertDimConstraints(x, "dimensions of x", RC_LENGTH | RC_EQ, rc_asRLength(2),
+    bool forceUpdate     = rc_getBool(forceUpdateExpr,         "forceUpdate", RC_NA | RC_NO, RC_END);
+    bool updateCutPoints = rc_getBool(updateCutPointsExpr, "updateCutPoints", RC_NA | RC_NO, RC_END);
+    
+    rc_assertDimConstraints(xExpr, "dimensions of x", RC_LENGTH | RC_EQ, rc_asRLength(2),
                             RC_VALUE | RC_EQ, static_cast<int>(fit->data.numObservations),
                             RC_VALUE | RC_EQ, static_cast<int>(fit->data.numPredictors),
                             RC_END);
+      
+    bool result = fit->setPredictor(REAL(xExpr), forceUpdate, updateCutPoints);
+        
+    return Rf_ScalarLogical(result);
+  }
+  
+  SEXP updatePredictor(SEXP fitExpr, SEXP xExpr, SEXP colsExpr, SEXP forceUpdateExpr, SEXP updateCutPointsExpr)
+  {
+    BARTFit* fit = static_cast<BARTFit*>(R_ExternalPtrAddr(fitExpr));
+    if (fit == NULL) Rf_error("dbarts_updatePredictor called on NULL external pointer");
     
-    // SEXP dimsExpr = Rf_getAttrib(x, R_DimSymbol);
-    //
-    //if (Rf_isNull(dimsExpr) || rc_getLength(dimsExpr) != 2) Rf_error("x must be a matrix, i.e. have two dimensions");
-    //int* dims = INTEGER(dimsExpr);
-    //
-    //if (static_cast<size_t>(dims[0]) != fit->data.numObservations) Rf_error("number of rows in new x does not match y");
-    //if (static_cast<size_t>(dims[1]) != fit->data.numPredictors) Rf_error("number of columns in new x does not match old");
+    if (!Rf_isReal(xExpr)) Rf_error("x must be of type real");
     
-    return Rf_ScalarLogical(fit->setPredictor(REAL(x)));
+    bool forceUpdate     = rc_getBool(forceUpdateExpr,         "forceUpdate", RC_NA | RC_NO, RC_END);
+    bool updateCutPoints = rc_getBool(updateCutPointsExpr, "updateCutPoints", RC_NA | RC_NO, RC_END);
+    
+    bool result;
+    if (Rf_isNull(colsExpr)) {
+      rc_assertDimConstraints(xExpr, "dimensions of x", RC_LENGTH | RC_EQ, rc_asRLength(2),
+                              RC_VALUE | RC_EQ, static_cast<int>(fit->data.numObservations),
+                              RC_VALUE | RC_EQ, static_cast<int>(fit->data.numPredictors),
+                              RC_END);
+      
+      result = fit->setPredictor(REAL(xExpr), forceUpdate, updateCutPoints);
+    } else {
+      
+      if (!Rf_isInteger(colsExpr)) Rf_error("columns must be of type integer");
+      
+      SEXP dimsExpr = Rf_getAttrib(xExpr, R_DimSymbol);
+      int* dims = NULL;
+      
+      if (!Rf_isNull(dimsExpr)) {
+        size_t numDims = rc_getLength(dimsExpr);
+        
+        if (numDims != 1 && numDims != 2) Rf_error("x must be a vector or a matrix");
+        if (numDims == 2) dims = INTEGER(dimsExpr);
+      }
+      
+      if (rc_getLength(colsExpr) == 0) Rf_error("length of columns is 0");
+      
+      if (dims != NULL) {
+        if (static_cast<size_t>(dims[0]) != fit->data.numObservations) Rf_error("number of rows of new x does not match y");
+        if (static_cast<size_t>(dims[1]) != rc_getLength(colsExpr)) Rf_error("number of columns of new x does not match length of columns to replace");
+      } else {
+        if (rc_getLength(xExpr) != fit->data.numObservations) Rf_error("length of new x does not match y");
+      }
+      
+      int* colsInt = INTEGER(colsExpr);
+      size_t numCols = rc_getLength(colsExpr);
+      size_t* cols = misc_stackAllocate(numCols, size_t);
+      for (size_t i = 0 ; i < numCols; ++i) {
+        cols[i] = static_cast<size_t>(colsInt[i] - 1);
+        if (cols[i] >= fit->data.numPredictors) {
+          misc_stackFree(cols);
+          Rf_error("column '%d' is out of range", colsInt[i] + 1);
+        }
+      }
+      
+      result = fit->updatePredictor(REAL(xExpr), cols, numCols, forceUpdate, updateCutPoints);
+      
+      misc_stackFree(cols);
+    }
+    
+    return Rf_ScalarLogical(result);
+  }
+  
+  SEXP setCutPoints(SEXP fitExpr, SEXP cutPointsExpr, SEXP colsExpr)
+  {
+    BARTFit* fit = static_cast<BARTFit*>(R_ExternalPtrAddr(fitExpr));
+    if (fit == NULL) Rf_error("dbarts_setCutPoints called on NULL external pointer");
+    
+    if (!Rf_isNewList(cutPointsExpr)) Rf_error("cutPoints must be of type list");
+ 
+    
+    size_t numCols;
+    if (Rf_isNull(colsExpr)) {
+      numCols = fit->data.numPredictors;
+    } else {
+      if (!Rf_isInteger(colsExpr)) Rf_error("columns must be of type integer");
+      numCols = rc_getLength(colsExpr);
+    }
+    
+    if (rc_getLength(cutPointsExpr) != numCols)
+      Rf_error("length of cutPoints (%lu) must equal length of columns (%lu)", rc_getLength(cutPointsExpr), numCols);
+    
+    const double** cutPoints = misc_stackAllocate(numCols, const double*);
+    uint32_t* numCutPoints = misc_stackAllocate(numCols, uint32_t);
+    size_t* cols = misc_stackAllocate(numCols, size_t);
+    
+    int* colsInt = Rf_isNull(colsExpr) ? NULL : INTEGER(colsExpr);
+    for (size_t i = 0; i < numCols; ++i) {
+      SEXP cutPointsExpr_i = VECTOR_ELT(cutPointsExpr, i);
+      
+      cutPoints[i] = REAL(cutPointsExpr_i);
+      numCutPoints[i] = static_cast<uint32_t>(rc_getLength(cutPointsExpr_i));
+      cols[i] = colsInt == NULL ? i : static_cast<size_t>(colsInt[i] - 1);
+       
+      if (cols[i] >= fit->data.numPredictors) {
+        misc_stackFree(cols);
+        misc_stackFree(numCutPoints);
+        misc_stackFree(cutPoints);
+        Rf_error("column '%d' is out of range", colsInt[i] + 1);
+      }
+    }
+    
+    fit->setCutPoints(cutPoints, numCutPoints, cols, numCols);
+    
+    misc_stackFree(cols);
+    misc_stackFree(numCutPoints);
+    misc_stackFree(cutPoints);
+        
+    return R_NilValue;
   }
   
   SEXP setTestPredictor(SEXP fitExpr, SEXP x_test)
@@ -375,11 +514,6 @@ extern "C" {
                             RC_END);
     int* dims = INTEGER(Rf_getAttrib(x_test, R_DimSymbol));
     
-    // SEXP dimsExpr = Rf_getAttrib(x_test, R_DimSymbol);
-    // if (rc_getLength(dimsExpr) != 2) Rf_error("x.test must be a matrix, i.e. have two dimensions");
-    // int* dims = INTEGER(dimsExpr);
-    // if (static_cast<size_t>(dims[1]) != fit->data.numPredictors) Rf_error("number of columns of x.test and x must be equal");
-    
     fit->setTestPredictor(REAL(x_test), static_cast<size_t>(dims[0]));
     
     return R_NilValue;
@@ -420,11 +554,6 @@ extern "C" {
                             RC_END);
     int* dims = INTEGER(Rf_getAttrib(x_test, R_DimSymbol));
     
-    // SEXP dimsExpr = Rf_getAttrib(x_test, R_DimSymbol);
-    // if (rc_getLength(dimsExpr) != 2) Rf_error("x.test must be a matrix, i.e. have two dimensions");
-    // int* dims = INTEGER(dimsExpr);
-    // if (static_cast<size_t>(dims[1]) != fit->data.numPredictors) Rf_error("number of columns of x.test and x must be equal");
-    
     if (Rf_isNull(offset_test)) {
       fit->setTestPredictorAndOffset(REAL(x_test), NULL, static_cast<size_t>(dims[0]));
     } else {
@@ -440,53 +569,6 @@ extern "C" {
     return R_NilValue;
   }
   
-  
-  SEXP updatePredictor(SEXP fitExpr, SEXP x, SEXP colsExpr)
-  {
-    BARTFit* fit = static_cast<BARTFit*>(R_ExternalPtrAddr(fitExpr));
-    if (fit == NULL) Rf_error("dbarts_updatePredictor called on NULL external pointer");
-    
-    if (!Rf_isReal(x)) Rf_error("x must be of type real");
-    if (!Rf_isInteger(colsExpr)) Rf_error("columns must be of type integer");
-    
-    SEXP dimsExpr = Rf_getAttrib(x, R_DimSymbol);
-    int* dims = NULL;
-    
-    if (!Rf_isNull(dimsExpr)) {
-      size_t numDims = rc_getLength(dimsExpr);
-      
-      if (numDims != 1 && numDims != 2) Rf_error("x must be a vector or a matrix");
-      if (numDims == 2) dims = INTEGER(dimsExpr);
-    }
-    
-    if (rc_getLength(colsExpr) == 0) Rf_error("length of columns is 0");
-
-    if (dims != NULL) {
-      if (static_cast<size_t>(dims[0]) != fit->data.numObservations) Rf_error("number of rows of new x does not match y");
-      if (static_cast<size_t>(dims[1]) != rc_getLength(colsExpr)) Rf_error("number of columns of new x does not match length of columns to replace");
-    } else {
-      if (rc_getLength(x) != fit->data.numObservations) Rf_error("length of new x does not match y");
-    }
-    
-    
-    int* colsInt = INTEGER(colsExpr);
-    size_t numCols = rc_getLength(colsExpr);
-    size_t* cols = ext_stackAllocate(numCols, size_t);
-    for (size_t i = 0 ; i < numCols; ++i) {
-      cols[i] = static_cast<size_t>(colsInt[i] - 1);
-      if (static_cast<size_t>(cols[i]) >= fit->data.numPredictors) {
-        ext_stackFree(cols);
-        Rf_error("column '%d' is out of range", colsInt[i]);
-      }
-    }
-    
-    bool result = fit->updatePredictors(REAL(x), cols, numCols);
-    
-    ext_stackFree(cols);
-    
-    return Rf_ScalarLogical(result);
-  }
-  
   SEXP updateTestPredictor(SEXP fitExpr, SEXP x_test, SEXP colsExpr)
   {
     BARTFit* fit = static_cast<BARTFit*>(R_ExternalPtrAddr(fitExpr));
@@ -516,21 +598,20 @@ extern "C" {
       if (rc_getLength(x_test) != fit->data.numTestObservations) Rf_error("length of new x does not match old x.test");
     }
     
-    
     int* colsInt = INTEGER(colsExpr);
     size_t numCols = rc_getLength(colsExpr);
-    size_t* cols = ext_stackAllocate(numCols, size_t);
+    size_t* cols = misc_stackAllocate(numCols, size_t);
     for (size_t i = 0 ; i < numCols; ++i) {
       cols[i] = static_cast<size_t>(colsInt[i] - 1);
       if (cols[i] >= fit->data.numPredictors) {
-        ext_stackFree(cols);
+        misc_stackFree(cols);
         Rf_error("column '%d' is out of range", colsInt[i]);
       }
     }
     
     fit->updateTestPredictors(REAL(x_test), cols, numCols);
     
-    ext_stackFree(cols);
+    misc_stackFree(cols);
     
     return R_NilValue;
   }
@@ -567,20 +648,27 @@ extern "C" {
   
   SEXP printTrees(SEXP fitExpr, SEXP chainIndicesExpr, SEXP sampleIndicesExpr, SEXP treeIndicesExpr)
   {
-    BARTFit* fit = static_cast<BARTFit*>(R_ExternalPtrAddr(fitExpr));
-    if (fit == NULL) Rf_error("dbarts_printTrees called on NULL external pointer");
+    BARTFit* fitPtr = static_cast<BARTFit*>(R_ExternalPtrAddr(fitExpr));
+    if (fitPtr == NULL) Rf_error("dbarts_printTrees called on NULL external pointer");
+    BARTFit& fit(*fitPtr);
     
-    size_t numChains  = fit->control.numChains;
-    size_t numSamples = fit->control.keepTrees ? fit->currentNumSamples : 0;
-    size_t numTrees   = fit->control.numTrees;
+    size_t numChains  = fit.control.numChains;
+    size_t numSamples = fit.control.keepTrees ? fit.currentNumSamples : 0;
+    size_t numTrees   = fit.control.numTrees;
     
     size_t numChainIndices  = Rf_isNull(chainIndicesExpr)  ? numChains  : rc_getLength(chainIndicesExpr);
     size_t numSampleIndices = Rf_isNull(sampleIndicesExpr) ? numSamples : rc_getLength(sampleIndicesExpr);
     size_t numTreeIndices   = Rf_isNull(treeIndicesExpr)   ? numTrees   : rc_getLength(treeIndicesExpr);
     
+    if (numChainIndices > numChains)
+      Rf_error("%lu chains specified but only %lu in sampler", numChainIndices, numChains);
+    if (numSampleIndices > numSamples)
+      Rf_error("%lu samples specified but only %lu in sampler", numSampleIndices, numSamples);
+    if (numTreeIndices > numTrees)
+      Rf_error("%lu trees specified but only %lu in sampler", numTreeIndices, numTrees);    
     
-    size_t* chainIndices  = ext_stackAllocate(numChainIndices, size_t);
-    size_t* sampleIndices = fit->control.keepTrees ? new size_t[numSamples] : NULL;
+    size_t* chainIndices  = misc_stackAllocate(numChainIndices, size_t);
+    size_t* sampleIndices = fit.control.keepTrees ? new size_t[numSamples] : NULL;
     size_t* treeIndices   = new size_t[numTreeIndices];
     
     if (Rf_isNull(chainIndicesExpr)) {
@@ -604,42 +692,160 @@ extern "C" {
       for (size_t i = 0; i < numTreeIndices; ++i) treeIndices[i] = static_cast<size_t>(i_treeIndices[i] - 1);
     }
    
-    fit->printTrees(chainIndices, numChainIndices, sampleIndices, numSampleIndices, treeIndices, numTreeIndices);
+    fit.printTrees(chainIndices, numChainIndices, sampleIndices, numSampleIndices, treeIndices, numTreeIndices);
     
     delete [] treeIndices;
     delete [] sampleIndices;
-    ext_stackFree(chainIndices);
-    
-    return R_NilValue;
-  }
-
-  
-  SEXP saveToFile(SEXP fitExpr, SEXP fileName)
-  {
-    BARTFit* fit = static_cast<BARTFit*>(R_ExternalPtrAddr(fitExpr));
-    if (fit == NULL) Rf_error("dbarts_saveToFile called on NULL external pointer");
-    
-    return Rf_ScalarLogical(fit->saveToFile(CHAR(STRING_ELT(fileName, 0))));
-  }
-
-  SEXP loadFromFile(SEXP fileName)
-  {
-    BARTFit* fit = BARTFit::loadFromFile(CHAR(STRING_ELT(fileName, 0)));
-    
-    delete [] fit->data.maxNumCuts;
-    delete [] fit->data.variableTypes;
-    
-    delete fit->model.sigmaSqPrior;
-    delete fit->model.muPrior;
-    delete fit->model.treePrior;
-    
-    delete fit;
-    
-    /* TODO: turn into an R object */
+    misc_stackFree(chainIndices);
     
     return R_NilValue;
   }
   
+  SEXP getTrees(SEXP fitExpr, SEXP chainIndicesExpr, SEXP sampleIndicesExpr, SEXP treeIndicesExpr)
+  {
+    BARTFit* fitPtr = static_cast<BARTFit*>(R_ExternalPtrAddr(fitExpr));
+    if (fitPtr == NULL) Rf_error("dbarts_getTrees called on NULL external pointer");
+    BARTFit& fit(*fitPtr);
+    
+    size_t numChains  = fit.control.numChains;
+    size_t numSamples = fit.control.keepTrees ? fit.currentNumSamples : 0;
+    size_t numTrees   = fit.control.numTrees;
+    
+    size_t numChainIndices  = Rf_isNull(chainIndicesExpr)  ? numChains  : rc_getLength(chainIndicesExpr);
+    size_t numSampleIndices = Rf_isNull(sampleIndicesExpr) ? numSamples : rc_getLength(sampleIndicesExpr);
+    size_t numTreeIndices   = Rf_isNull(treeIndicesExpr)   ? numTrees   : rc_getLength(treeIndicesExpr);
+    
+    if (numChainIndices > numChains)
+      Rf_error("%lu chains specified but only %lu in sampler", numChainIndices, numChains);
+    if (numSampleIndices > numSamples)
+      Rf_error("%lu samples specified but only %lu in sampler", numSampleIndices, numSamples);
+    if (numTreeIndices > numTrees)
+      Rf_error("%lu trees specified but only %lu in sampler", numTreeIndices, numTrees);
+    
+    size_t* chainIndices  = misc_stackAllocate(numChainIndices, size_t);
+    size_t* sampleIndices = fit.control.keepTrees ? new size_t[numSamples] : NULL;
+    size_t* treeIndices   = new size_t[numTreeIndices];
+    
+    if (Rf_isNull(chainIndicesExpr)) {
+      for (size_t i = 0; i < numChains; ++i) chainIndices[i] = i;
+    } else {
+      int* i_chainIndices = INTEGER(chainIndicesExpr);
+      for (size_t i = 0; i < numChainIndices; ++i) chainIndices[i] = static_cast<size_t>(i_chainIndices[i] - 1);
+    }
+    
+    if (Rf_isNull(sampleIndicesExpr)) {
+      for (size_t i = 0; i < numSamples; ++i) sampleIndices[i] = i;
+    } else {
+      int* i_sampleIndices = INTEGER(sampleIndicesExpr);
+      for (size_t i = 0; i < numSampleIndices; ++i) sampleIndices[i] = static_cast<size_t>(i_sampleIndices[i] - 1);
+    }
+    
+    if (Rf_isNull(treeIndicesExpr)) {
+      for (size_t i = 0; i < numTrees; ++i) treeIndices[i] = i;
+    } else {
+      int* i_treeIndices = INTEGER(treeIndicesExpr);
+      for (size_t i = 0; i < numTreeIndices; ++i) treeIndices[i] = static_cast<size_t>(i_treeIndices[i] - 1);
+    }
+    
+    FlattenedTrees* flattenedTreesPtr =
+      fit.getFlattenedTrees(chainIndices, numChainIndices,
+                            sampleIndices, numSampleIndices,
+                            treeIndices, numTreeIndices);
+    FlattenedTrees& flattenedTrees(*flattenedTreesPtr);
+    
+    delete [] treeIndices;
+    delete [] sampleIndices;
+    misc_stackFree(chainIndices);
+    
+    R_xlen_t numCols = 4 + (numChains > 1 ? 1 : 0) + (fit.control.keepTrees ? 1 : 0);
+    SEXP resultExpr = PROTECT(rc_newList(numCols));
+        
+    SEXP classExpr = PROTECT(rc_newCharacter(1));
+    SET_STRING_ELT(classExpr, 0, Rf_mkChar("data.frame"));
+    Rf_setAttrib(resultExpr, R_ClassSymbol, classExpr);
+    UNPROTECT(1);
+    
+    SEXP resultRowNamesExpr;
+    rc_allocateInSlot2(resultRowNamesExpr, resultExpr, R_RowNamesSymbol, STRSXP, flattenedTrees.totalNumNodes);
+    
+    SEXP resultNamesExpr;
+    rc_allocateInSlot2(resultNamesExpr, resultExpr, R_NamesSymbol, STRSXP, numCols);
+    
+    int* chainNumber = NULL;
+    int* sampleNumber = NULL;
+    int* treeNumber, *numObservations, *variable;
+    double* value;
+        
+    R_xlen_t colNum = 0;
+    if (numChains > 1) {
+      SET_VECTOR_ELT(resultExpr, colNum, PROTECT(rc_newInteger(flattenedTrees.totalNumNodes)));
+      SET_STRING_ELT(resultNamesExpr, colNum, PROTECT(Rf_mkChar("chain")));
+      UNPROTECT(2);
+      chainNumber = INTEGER(VECTOR_ELT(resultExpr, colNum));
+      ++colNum;
+    }
+    if (fit.control.keepTrees) {
+      SET_VECTOR_ELT(resultExpr, colNum, PROTECT(rc_newInteger(flattenedTrees.totalNumNodes)));
+      SET_STRING_ELT(resultNamesExpr, colNum, PROTECT(Rf_mkChar("sample")));
+      UNPROTECT(2);
+      sampleNumber = INTEGER(VECTOR_ELT(resultExpr, colNum));
+      ++colNum;
+    }
+    SET_VECTOR_ELT(resultExpr, colNum, PROTECT(rc_newInteger(flattenedTrees.totalNumNodes)));
+    SET_STRING_ELT(resultNamesExpr, colNum, PROTECT(Rf_mkChar("tree")));
+    treeNumber = INTEGER(VECTOR_ELT(resultExpr, colNum));
+    ++colNum;
+    SET_VECTOR_ELT(resultExpr, colNum, PROTECT(rc_newInteger(flattenedTrees.totalNumNodes)));
+    SET_STRING_ELT(resultNamesExpr, colNum, PROTECT(Rf_mkChar("n")));
+    numObservations = INTEGER(VECTOR_ELT(resultExpr, colNum));
+    ++colNum;
+    SET_VECTOR_ELT(resultExpr, colNum, PROTECT(rc_newInteger(flattenedTrees.totalNumNodes)));
+    SET_STRING_ELT(resultNamesExpr, colNum, PROTECT(Rf_mkChar("var")));
+    variable = INTEGER(VECTOR_ELT(resultExpr, colNum));
+    ++colNum;
+    SET_VECTOR_ELT(resultExpr, colNum, PROTECT(rc_newReal(flattenedTrees.totalNumNodes)));
+    SET_STRING_ELT(resultNamesExpr, colNum, PROTECT(Rf_mkChar("value")));
+    value = REAL(VECTOR_ELT(resultExpr, colNum));
+    UNPROTECT(8);
+    
+    size_t numDigits = 1;
+    size_t temp = flattenedTrees.totalNumNodes;
+    while (temp >= 10) {
+      temp /= 10;
+      ++numDigits;
+    }
+    char* buffer = new char[numDigits + 1];
+    for (size_t i = 0; i < flattenedTrees.totalNumNodes; ++i) {
+      if (chainNumber != NULL)
+        chainNumber[i] = static_cast<int>(flattenedTrees.chainNumber[i] + 1);
+      if (sampleNumber != NULL)
+        sampleNumber[i] = static_cast<int>(flattenedTrees.sampleNumber[i] + 1);
+      treeNumber[i] = static_cast<int>(flattenedTrees.treeNumber[i] + 1);
+      numObservations[i] = static_cast<int>(flattenedTrees.numObservations[i]);
+      int variable_i = static_cast<int>(flattenedTrees.variable[i]);
+      variable[i] = variable_i >= 0 ? variable_i + 1 : variable_i;
+      value[i] = flattenedTrees.value[i];
+#if defined(__MINGW32__) && __cplusplus < 201112L
+#  ifdef _WIN64
+      std::sprintf(buffer, "%lu", static_cast<unsigned long>(i + 1));
+#  else
+      std::sprintf(buffer, "%u", i + 1);
+#  endif
+#else
+      std::sprintf(buffer, "%zu", i + 1);
+#endif
+      SET_STRING_ELT(resultRowNamesExpr, i, PROTECT(Rf_mkChar(buffer)));
+      UNPROTECT(1);
+    }
+    
+    delete [] buffer;
+    delete flattenedTreesPtr;
+    
+    UNPROTECT(1);
+    
+    return resultExpr;
+  }
+
   
   static void fitFinalizer(SEXP fitExpr)
   {
