@@ -3,6 +3,7 @@
 
 #include <cstddef>
 #include <dbarts/cstdint.hpp>
+#include <dbarts/types.hpp>
 
 #include "node.hpp"
 
@@ -10,20 +11,23 @@ extern "C" struct ext_rng;
 
 namespace dbarts {
   struct BARTFit;
+  struct SavedTree;
   
   struct Tree {
     Node top;
     
     Tree(std::size_t* indices, std::size_t numObservations, std::size_t numPredictors) : top(indices, numObservations, numPredictors) { }
-    void copyFrom(const BARTFit& fit, const Tree& other);
     
-    void sampleAveragesAndSetFits(const BARTFit& fit, std::size_t chainNum, double sigma, double* trainingFits, double* testFits);
-    double* recoverAveragesFromFits(const BARTFit& fit, const double* treeFits); // allocates result; are ordered as bottom nodes are
-    void setCurrentFitsFromAverages(const BARTFit& fit, const double* posteriorPredictions, double* trainingFits, double* testFits);
-    void setCurrentFitsFromAverages(const BARTFit& fit, const double* posteriorPredictions, const double* xt, std::size_t numObservations, double* fits);
+    void sampleParametersAndSetFits(const BARTFit& fit, std::size_t chainNum, double sigma, double* trainingFits, double* testFits);
+    double* recoverParametersFromFits(const BARTFit& fit, const double* treeFits); // allocates result; are ordered as bottom nodes are
+    double* recoverParametersFromFits(const BARTFit& fit, const double* treeFits, std::size_t* numBottomNodes); // allocates result; are ordered as bottom nodes are
+    void setCurrentFitsFromParameters(const BARTFit& fit, const double* nodeParams, double* trainingFits, double* testFits);
+    void setCurrentFitsFromParameters(const BARTFit& fit, const double* nodeParams, const xint_t* xt, std::size_t numObservations, double* fits);
     
-    void mapOldCutPointsOntoNew(const BARTFit& fit, const double* const* oldCutPoints, double* posteriorPredictions);
-    void collapseEmptyNodes(const BARTFit& fit, double* posteriorPredictions);
+    // deals largely if there are a different number of cut points, since a tree could then conceivably have
+    // splits out of range
+    void mapOldCutPointsOntoNew(const BARTFit& fit, const double* const* oldCutPoints, double* nodeParams);
+    void collapseEmptyNodes(const BARTFit& fit, double* nodeParams);
     
     void sampleFromPrior(const BARTFit& fit, ext_rng* rng);
     
@@ -40,15 +44,32 @@ namespace dbarts {
     NodeVector getNodesWhoseChildrenAreAtBottom() const;
     NodeVector getSwappableNodes() const;
     
-    void setNodeAverages(const BARTFit& fit, std::size_t chainNUm, const double* y);
+    void setNodeAverages(const BARTFit& fit, std::size_t chainNum, const double* y);
     
     void countVariableUses(std::uint32_t* variableCounts) const;
     
     const char* createString() const;
     
+    std::size_t getSerializedLength(const BARTFit& fit) const;
+    std::size_t serialize(const BARTFit& fit, void* state) const;
+    std::size_t deserialize(const BARTFit& fit, const void* state);
+    
     bool isValid() const;
   };
   
+  struct SavedTree {
+    SavedNode top;
+    
+    SavedTree() : top() { }
+    void copyStructureFrom(const BARTFit& fit, const Tree& other, const double* treeFits);
+    
+    void getPredictions(const BARTFit& fit, const double* xt, std::size_t numTestObservations, double* result);
+    
+    std::size_t getSerializedLength() const;
+    std::size_t serialize(void* state) const;
+    std::size_t deserialize(const void* state);
+  };
+  
   
   
   
